-- Local Table
local aName, aTable = ...

-- Locals
local inLockdown
local stones = {
54452, -- Ethereal Portal
64488, -- Innkeeper's Daughter
93672, -- Dark Portal
142542, -- Tome of Town Portal
162973, -- Greatfather Winter's Hearthstone
163045, -- Headless Horseman's Hearthstone
163206, -- Weary Spirit Binding (Not implemented?)
165669, -- Lunar Elder's Hearthstone
165670, -- Peddlefeet's Lovely Hearthstone
165802, -- Noble Gardener's Hearthstone
166746, -- Fire Eater's Hearthstone
166747, -- Brewfest Reveler's Hearthstone
168907, -- Holographic Digitalization Hearthstone
}

-- OnEvent
function aTable.OnEvent(self, event, ...)
	if ( event == "ADDON_LOADED" ) then
		local addonName = ...
		if ( not addonName or (addonName and addonName ~= "RandomHearth") ) then
			return;
		end

		if RandomHearth == nil then
			RandomHearth = {}
		end

		if RandomHearth["hasStones"] == nil then
			RandomHearth["hasStones"] = {}
		end
	elseif event == ("PLAYER_LOGIN" or "NEW_TOY_ADDED") then
		aTable.UpdateStones()
	elseif event == ("PLAYER_ENTERING_WORLD" or ("PLAYER_REGEN_ENABLED" and inLockdown))then
		aTable.UpdateMacro()
	elseif event == "GET_ITEM_INFO_RECEIVED" then
		local tempWait = ...
		if itemWait == tempWait then
			tempWait = GetItemInfo(itemWait)
			RandomHearth.hasStones[itemWait] = tempWait
		end
	end
end

function aTable.UpdateStones()
	for i = 1, #stones do
		if PlayerHasToy(stones[i]) then
			if not RandomHearth.hasStones[(stones[i])] then
				toyName = GetItemInfo(stones[i])
				if toyName then
					RandomHearth.hasStones[stones[i]] = toyName
				else
					itemWait = stones[i]
				end
			end
		elseif RandomHearth.hasStones[stones[i]] then
			RandomHearth.hasStones[stones[i]] = nil
		end
	end
end

function aTable.UpdateMacro()
	if InCombatLockdown() then inLockdown = 1 return end

	local randStone = aTable.RandomStone()
	inLockdown = nil

	if GetMacroInfo("RH") ~= nil then
		if randStone then
			EditMacro("RH", "RH", "INV_Misc_QuestionMark", "#showtooltip\n/usetoy "..randStone.."")
		elseif GetItemCount(6948) == 1 then
			EditMacro("RH", "RH", "INV_Misc_QuestionMark", "#showtooltip\n/use item:6948");
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cff20ff20RandomHearth: You don't appear to have any way of hearthing.")
		end
	else
		if GetNumMacros() == 36 then
			DEFAULT_CHAT_FRAME:AddMessage("|cff20ff20RandomHearth: No available global macro slots.")
			return
		end

		if randStone then
			CreateMacro("RH", "INV_Misc_QuestionMark", "#showtooltip\n/usetoy "..randStone.."", nil, 1);
		elseif GetItemCount(6948) == 1 then
			CreateMacro("RH", "INV_Misc_QuestionMark", "#showtooltip\n/use item:6948", nil, 1);
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cff20ff20RandomHearth: You don't appear to have any way of hearthing.")
		end
	end
end

function aTable.RandomStone()
	local randStone = {}
	for k in pairs(RandomHearth["hasStones"]) do
		tinsert(randStone, k)
	end

	if #randStone > 0 then
		return RandomHearth.hasStones[randStone[random(#randStone)]]
	else
		return nil
	end
end

-- Local event frame
local f = CreateFrame("Frame")
f:RegisterEvent("ADDON_LOADED")
f:RegisterEvent("PLAYER_ENTERING_WORLD")
f:RegisterEvent("PLAYER_LOGIN")
f:RegisterEvent("GET_ITEM_INFO_RECEIVED")
f:RegisterEvent("NEW_TOY_ADDED")
f:RegisterEvent("PLAYER_REGEN_ENABLED")
f:SetScript("OnEvent", aTable.OnEvent)
